/*******************************************************************************
				Neoinput
				========

		Copyright (C) 2010 Paul Kocialkowski

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>

#include <linux/input.h>
#include <linux/uinput.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>

#include "client.h"
#include "input_read.h"
#include "input.h"

#include "input_write.h"

int input=0;
char *input_device=NULL;
float speed_factor=1;

char *uinput_devices[3]={"/dev/input/uinput/", "/dev/uinput", "/dev/misc/uinput"};

void input_open_write(int input_type)
{
	struct uinput_user_dev uinput_device;
	int i;

	/* Use uinput if nothing specified */
	if(input_device == NULL)
	{
		/* Try each uinput device */
		for(i=0 ; i < 2 ; i++)
		{
			input=open(uinput_devices[i], O_WRONLY | O_NONBLOCK);

			if(input < 0)
				continue;
			else
				break;
		}
	}
	else
		input=open(input_device, O_WRONLY);

	if(input < 0)
	{
		if(input_device == NULL)
			printf("Error while opening uinput.\n");
		else
			printf("Error while opening %s.\n", input_device);
		exit(1);
	}
	printf("input is %s\n", uinput_devices[i]);
	switch(input_type)
	{
		case INPUT_TYPE_TOUCH:
		case INPUT_TYPE_TOUCH_ROTATE:
		case INPUT_TYPE_MULTIMEDIA:

			ioctl(input, UI_SET_EVBIT, EV_KEY);
			ioctl(input, UI_SET_KEYBIT, BTN_LEFT);
			ioctl(input, UI_SET_KEYBIT, BTN_RIGHT);

			ioctl(input, UI_SET_EVBIT, EV_REL); 
			ioctl(input, UI_SET_RELBIT, REL_X);
			ioctl(input, UI_SET_RELBIT, REL_Y);

			ioctl(input, UI_SET_EVBIT, EV_SYN);
/*
			if(input_type == INPUT_TYPE_MULTIMEDIA)
				//ioctl les touches que l'on utilise…
*/

			if(input_device == NULL)
			{
				memset(&uinput_device, 0, sizeof(uinput_device));
				snprintf(uinput_device.name, UINPUT_MAX_NAME_SIZE, "uinput-sample");
				uinput_device.id.bustype = BUS_USB;
				uinput_device.id.vendor  = 0x1;
				uinput_device.id.product = 0x1;
				uinput_device.id.version = 1;

				write(input, &uinput_device, sizeof(uinput_device));

				ioctl(input, UI_DEV_CREATE);
			}
		break;
	}
}

void input_event_write(struct input_event data)
{
	if(!write(input, &data, sizeof(struct input_event)))
	{
		perror("Error in write()");
		exit(1);
	}
}

void input_close(void)
{
	if(input_device == NULL)
		ioctl(input, UI_DEV_DESTROY);
	close(input);
}
